@@ -24,22 +24,33 @@
 
 #include <algorithm>
 
-static PixelFormat CSNameToPIXFMT(const char *CSName, PixelFormat Default) {
+static AVPixelFormat CSNameToPIXFMT(const char *CSName, AVPixelFormat Default) {
 	if (!CSName)
-		return PIX_FMT_NONE;
+		return FFMS_PIX_FMT(NONE);
 	std::string s = CSName;
 	std::transform(s.begin(), s.end(), s.begin(), toupper);
 	if (s == "")
 		return Default;
+	if (s == "YUV9")
+		return FFMS_PIX_FMT(YUV410P);
+	if (s == "YV411")
+		return FFMS_PIX_FMT(YUV411P);
 	if (s == "YV12")
-		return PIX_FMT_YUV420P;
+		return FFMS_PIX_FMT(YUV420P);
+	if (s == "YV16")
+		return FFMS_PIX_FMT(YUV422P);
+	if (s == "YV24")
+		return FFMS_PIX_FMT(YUV444P);
+	if (s == "Y8")
+		return FFMS_PIX_FMT(GRAY8);
 	if (s == "YUY2")
-		return PIX_FMT_YUYV422;
+		return FFMS_PIX_FMT(YUYV422);
 	if (s == "RGB24")
-		return PIX_FMT_BGR24;
+		return FFMS_PIX_FMT(BGR24);
 	if (s == "RGB32")
-		return PIX_FMT_RGB32;
-	return PIX_FMT_NONE;
+		return FFMS_PIX_FMT(RGB32);
+
+	return FFMS_PIX_FMT(NONE);
 }
 
 AvisynthVideoSource::AvisynthVideoSource(const char *SourceFile, int Track, FFMS_Index *Index,
@@ -197,18 +208,23 @@ void AvisynthVideoSource::InitOutputFormat(
 	if (!F)
 		Env->ThrowError("FFVideoSource: %s", E.Buffer);
 
-	int TargetFormats[4];
-	TargetFormats[0] = FFMS_GetPixFmt("yuv420p");
-	TargetFormats[1] = FFMS_GetPixFmt("yuyv422");
-	TargetFormats[2] = FFMS_GetPixFmt("bgra");
-	TargetFormats[3] = -1;
+	int TargetFormats[9];
+	TargetFormats[0] = FFMS_GetPixFmt("yuv410p");
+	TargetFormats[1] = FFMS_GetPixFmt("yuv411p");
+	TargetFormats[2] = FFMS_GetPixFmt("yuv420p");
+	TargetFormats[3] = FFMS_GetPixFmt("yuv422p");
+	TargetFormats[4] = FFMS_GetPixFmt("yuv444p");
+	TargetFormats[5] = FFMS_GetPixFmt("gray8");
+	TargetFormats[6] = FFMS_GetPixFmt("yuyv422");
+	TargetFormats[7] = FFMS_GetPixFmt("bgra");
+	TargetFormats[8] = -1;
 
 	// PIX_FMT_NV21 is misused as a return value different to the defined ones in the function
-	PixelFormat TargetPixelFormat = CSNameToPIXFMT(ConvertToFormatName, PIX_FMT_NV21);
-	if (TargetPixelFormat == PIX_FMT_NONE)
+	AVPixelFormat TargetPixelFormat = CSNameToPIXFMT(ConvertToFormatName, FFMS_PIX_FMT(NV21));
+	if (TargetPixelFormat == FFMS_PIX_FMT(NONE))
 		Env->ThrowError("FFVideoSource: Invalid colorspace name specified");
 
-	if (TargetPixelFormat != PIX_FMT_NV21) {
+	if (TargetPixelFormat != FFMS_PIX_FMT(NV21)) {
 		TargetFormats[0] = TargetPixelFormat;
 		TargetFormats[1] = -1;
 	}
@@ -240,6 +256,16 @@ void AvisynthVideoSource::InitOutputFormat(
 
 	if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuvj420p") || F->ConvertedPixelFormat == FFMS_GetPixFmt("yuv420p"))
 		VI.pixel_type = VideoInfo::CS_I420;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuvj422p") || F->ConvertedPixelFormat == FFMS_GetPixFmt("yuv422p"))
+        VI.pixel_type = VideoInfo::CS_YV16;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuvj444p") || F->ConvertedPixelFormat == FFMS_GetPixFmt("yuv444p"))
+        VI.pixel_type = VideoInfo::CS_YV24;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuv411p"))
+        VI.pixel_type = VideoInfo::CS_YV411;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuv410p"))
+        VI.pixel_type = VideoInfo::CS_YUV9;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("gray8"))
+        VI.pixel_type = VideoInfo::CS_Y8;
 	else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuyv422"))
 		VI.pixel_type = VideoInfo::CS_YUY2;
 	else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("rgb32"))
@@ -252,7 +278,7 @@ void AvisynthVideoSource::InitOutputFormat(
 	if (RFFMode > 0 && ResizeToHeight != F->EncodedHeight)
 		Env->ThrowError("FFVideoSource: Vertical scaling not allowed in RFF mode");
 
-	if (RFFMode > 0 && TargetPixelFormat != PIX_FMT_NV21)
+	if (RFFMode > 0 && TargetPixelFormat != FFMS_PIX_FMT(NV21))
 		Env->ThrowError("FFVideoSource: Only the default output colorspace can be used in RFF mode");
 
 	// set color information variables
