@@ -1,4 +1,4 @@
-//  Copyright (c) 2012 Fredrik Mellbin
+//  Copyright (c) 2012-2015 Fredrik Mellbin
 //
 //  Permission is hereby granted, free of charge, to any person obtaining a copy
 //  of this software and associated documentation files (the "Software"), to deal
@@ -23,6 +23,10 @@
 
 extern "C" {
 #include <libavutil/avutil.h>
+#include <libavutil/common.h>
+#include <libavutil/pixdesc.h>
+#include <libavcodec/avcodec.h>
+#include <libswscale/swscale.h>
 }
 
 #include "VapourSynth.h"
@@ -31,23 +35,23 @@ extern "C" {
 
 struct VSVideoSource {
 private:
-	VSVideoInfo VI;
+	VSVideoInfo VI[2];
 	FFMS_VideoSource *V;
-	int FPSNum;
-	int FPSDen;
+	int64_t FPSNum;
+	int64_t FPSDen;
 	int SARNum;
 	int SARDen;
+	bool OutputAlpha;
 
 	void InitOutputFormat(int ResizeToWidth, int ResizeToHeight,
 		const char *ResizerName, int ConvertToFormat, const VSAPI *vsapi, VSCore *core);
-	static void OutputFrame(const FFMS_Frame *Frame, VSFrameRef *Dst, const VSAPI *vsapi, VSCore *core);
+	static void OutputFrame(const FFMS_Frame *Frame, VSFrameRef *Dst, const VSAPI *vsapi);
+	static void OutputAlphaFrame(const FFMS_Frame *Frame, int Plane, VSFrameRef *Dst, const VSAPI *vsapi);
 public:
-
-	const VSVideoInfo *GetVI() { return &VI; }
 	VSVideoSource(const char *SourceFile, int Track, FFMS_Index *Index,
-		int FPSNum, int FPSDen, int Threads, int SeekMode, int RFFMode,
+		int AFPSNum, int AFPSDen, int Threads, int SeekMode, int RFFMode,
 		int ResizeToWidth, int ResizeToHeight, const char *ResizerName,
-		int Format, const VSAPI *vsapi, VSCore *core);
+		int Format, bool OutputAlpha, const VSAPI *vsapi, VSCore *core);
 	~VSVideoSource();
 
 	static void VS_CC Init(VSMap *in, VSMap *out, void **instanceData, VSNode *node, VSCore *core, const VSAPI *vsapi);
