@@ -1,4 +1,4 @@
-//  Copyright (c) 2012 Fredrik Mellbin
+//  Copyright (c) 2012-2015 Fredrik Mellbin
 //
 //  Permission is hereby granted, free of charge, to any person obtaining a copy
 //  of this software and associated documentation files (the "Software"), to deal
@@ -20,15 +20,13 @@
 
 #include "ffms.h"
 #include "vapoursource.h"
+#include "VSHelper.h"
+#include "../core/utils.h"
 
 #include <cstdio>
 #include <cstring>
 #include <string>
-
-// assume windows is the only OS with a case insensitive filesystem
-#ifndef _WIN32
-#define _stricmp strcmp
-#endif
+#include <set>
 
 static void VS_CC CreateIndex(const VSMap *in, VSMap *out, void *, VSCore *, const VSAPI *vsapi)  {
 	FFMS_Init(0,  1);
@@ -39,16 +37,35 @@ static void VS_CC CreateIndex(const VSMap *in, VSMap *out, void *, VSCore *, con
 	E.BufferSize = sizeof(ErrorMsg);
 	int err;
 
-	const char *Source = vsapi->propGetData(in, "source", 0, 0);
+	std::set<int> IndexTracks;
+	std::set<int> DumpTracks;
+
+	const char *Source = vsapi->propGetData(in, "source", 0, nullptr);
 	const char *CacheFile = vsapi->propGetData(in, "cachefile", 0, &err);
-	int IndexMask = (int)vsapi->propGetInt(in, "indexmask", 0, &err);
-	if (err)
-		IndexMask = -1;
-	int DumpMask = (int)vsapi->propGetInt(in, "dumpmask", 0, &err);
+	
+	int NumIndexTracks = vsapi->propNumElements(in, "indextracks");
+	bool IndexAllTracks = (NumIndexTracks == 1) && (int64ToIntS(vsapi->propGetInt(in, "indextracks", 0, nullptr)) == -1);
+	if (!IndexAllTracks) {
+		for (int i = 0; i < NumIndexTracks; i++) {
+			int Track = int64ToIntS(vsapi->propGetInt(in, "indextracks", i, nullptr));
+			IndexTracks.insert(Track);
+		}
+	}
+
+	int NumDumpTracks = vsapi->propNumElements(in, "dumptracks");
+	bool DumpAllTracks = (NumDumpTracks == 1) && (int64ToIntS(vsapi->propGetInt(in, "dumptracks", 0, nullptr)) == -1);
+	if (!DumpAllTracks) {
+		for (int i = 0; i < NumDumpTracks; i++) {
+			int Track = int64ToIntS(vsapi->propGetInt(in, "dumptracks", i, nullptr));
+			IndexTracks.insert(Track);
+			DumpTracks.insert(Track);
+		}
+	}
+
 	const char *AudioFile = vsapi->propGetData(in, "audiofile", 0, &err);
 	if (err)
 		AudioFile = "%sourcefile%.%trackzn%.w64";
-	int ErrorHandling = (int)vsapi->propGetInt(in, "errorhandling", 0, &err);
+	int ErrorHandling = int64ToIntS(vsapi->propGetInt(in, "errorhandling", 0, &err));
 	if (err)
 		ErrorHandling = FFMS_IEH_IGNORE;
 	bool OverWrite = !!vsapi->propGetInt(in, "overwrite", 0, &err);
@@ -67,22 +84,32 @@ static void VS_CC CreateIndex(const VSMap *in, VSMap *out, void *, VSCore *, con
 	if (DemuxerStr) {
 		if (!strcmp(DemuxerStr, "lavf"))
 			Demuxer = FFMS_SOURCE_LAVF;
-		else if (!strcmp(DemuxerStr, "matroska"))
-			Demuxer = FFMS_SOURCE_MATROSKA;
-		else if (!strcmp(DemuxerStr, "haalimpeg"))
-			Demuxer = FFMS_SOURCE_HAALIMPEG;
-		else if (!strcmp(DemuxerStr, "haaliogg"))
-			Demuxer = FFMS_SOURCE_HAALIOGG;
 		else
 			return vsapi->setError(out, "Index: Invalid demuxer requested");
 	}
 
 	FFMS_Index *Index = FFMS_ReadIndex(CacheFile, &E);
-	if (OverWrite || !Index || (Index && FFMS_IndexBelongsToFile(Index, Source, 0) != FFMS_ERROR_SUCCESS)) {
+	if (OverWrite || !Index || (Index && FFMS_IndexBelongsToFile(Index, Source, nullptr) != FFMS_ERROR_SUCCESS)) {
 		FFMS_Indexer *Indexer = FFMS_CreateIndexerWithDemuxer(Source, Demuxer, &E);
-		if (!Indexer)
+		if (!Indexer) {
+			FFMS_DestroyIndex(Index);
 			return vsapi->setError(out, (std::string("Index: ") + E.Buffer).c_str());
-		if (!(Index = FFMS_DoIndexing(Indexer, IndexMask, DumpMask, FFMS_DefaultAudioFilename, (void *)AudioFile, ErrorHandling, NULL, NULL, &E)))
+		}
+
+		FFMS_SetAudioNameCallback(Indexer, FFMS_DefaultAudioFilename, (void *)AudioFile);
+
+		if (DumpAllTracks) {
+			FFMS_TrackTypeIndexSettings(Indexer, FFMS_TYPE_AUDIO, 1, 1);
+		} else if (IndexAllTracks) {
+			FFMS_TrackTypeIndexSettings(Indexer, FFMS_TYPE_AUDIO, 1, 0);
+			for (int i : DumpTracks)
+				FFMS_TrackIndexSettings(Indexer, i, 1, 1);
+		} else {
+			for (int i : IndexTracks)
+				FFMS_TrackIndexSettings(Indexer, i, 1, static_cast<int>(DumpTracks.count(i)));
+		}
+
+		if (!(Index = FFMS_DoIndexing2(Indexer, ErrorHandling, &E)))
 			return vsapi->setError(out, (std::string("Index: ") + E.Buffer).c_str());
 		if (FFMS_WriteIndex(CacheFile, Index, &E)) {
 			FFMS_DestroyIndex(Index);
@@ -90,12 +117,12 @@ static void VS_CC CreateIndex(const VSMap *in, VSMap *out, void *, VSCore *, con
 		}
 		FFMS_DestroyIndex(Index);
 		if (!OverWrite)
-			vsapi->propSetData(out, "result", "Index generated", 0, 0);
+			vsapi->propSetData(out, "result", "Index generated", -1, paReplace);
 		else
-			vsapi->propSetData(out, "result", "Index generated (forced overwrite)", 0, 0);
+			vsapi->propSetData(out, "result", "Index generated (forced overwrite)", -1, paReplace);
 	} else {
 		FFMS_DestroyIndex(Index);
-		vsapi->propSetData(out, "result", "Valid index already exists", 0, 0);
+		vsapi->propSetData(out, "result", "Valid index already exists", -1, paReplace);
 	}
 }
 
@@ -108,32 +135,36 @@ static void VS_CC CreateSource(const VSMap *in, VSMap *out, void *, VSCore *core
 	E.BufferSize = sizeof(ErrorMsg);
 	int err;
 
-	const char *Source = vsapi->propGetData(in, "source", 0, 0);
-	int Track = (int)vsapi->propGetInt(in, "track", 0, &err);
+	const char *Source = vsapi->propGetData(in, "source", 0, nullptr);
+	int Track = int64ToIntS(vsapi->propGetInt(in, "track", 0, &err));
 	if (err)
 		Track = -1;
 	bool Cache = !!vsapi->propGetInt(in, "cache", 0, &err);
 	if (err)
 		Cache = true;
 	const char *CacheFile = vsapi->propGetData(in, "cachefile", 0, &err);
-	int FPSNum = (int)vsapi->propGetInt(in, "fpsnum", 0, &err);
+	int FPSNum = int64ToIntS(vsapi->propGetInt(in, "fpsnum", 0, &err));
 	if (err)
 		FPSNum = -1;
-	int FPSDen = (int)vsapi->propGetInt(in, "fpsden", 0, &err);
+	int FPSDen = int64ToIntS(vsapi->propGetInt(in, "fpsden", 0, &err));
 	if (err)
 		FPSDen = 1;
-	int Threads = (int)vsapi->propGetInt(in, "threads", 0, &err);
+	int Threads = int64ToIntS(vsapi->propGetInt(in, "threads", 0, &err));
 	const char *Timecodes = vsapi->propGetData(in, "timecodes", 0, &err);
-	int SeekMode = (int)vsapi->propGetInt(in, "seekmode", 0, &err);
+	int SeekMode = int64ToIntS(vsapi->propGetInt(in, "seekmode", 0, &err));
 	if (err)
 		SeekMode = FFMS_SEEK_NORMAL;
-	int RFFMode = (int)vsapi->propGetInt(in, "rffmode", 0, &err);
-	int Width = (int)vsapi->propGetInt(in, "width", 0, &err);
-	int Height = (int)vsapi->propGetInt(in, "height", 0, &err);
+	int RFFMode = int64ToIntS(vsapi->propGetInt(in, "rffmode", 0, &err));
+	int Width = int64ToIntS(vsapi->propGetInt(in, "width", 0, &err));
+	int Height = int64ToIntS(vsapi->propGetInt(in, "height", 0, &err));
 	const char *Resizer = vsapi->propGetData(in, "resizer", 0, &err);
 	if (err)
 		Resizer = "BICUBIC";
-	int Format = (int)vsapi->propGetInt(in, "format", 0, &err);
+	int Format = int64ToIntS(vsapi->propGetInt(in, "format", 0, &err));
+
+	bool OutputAlpha = !!vsapi->propGetInt(in, "alpha", 0, &err);
+	if (err)
+		OutputAlpha = true;
 
 	if (FPSDen < 1)
 		return vsapi->setError(out, "Source: FPS denominator needs to be 1 or higher");
@@ -145,14 +176,14 @@ static void VS_CC CreateSource(const VSMap *in, VSMap *out, void *, VSCore *core
 		return vsapi->setError(out, "Source: Invalid RFF mode selected");
 	if (RFFMode > 0 && FPSNum > 0)
 		return vsapi->setError(out, "Source: RFF modes may not be combined with CFR conversion");
-	if (Timecodes && !_stricmp(Source, Timecodes))
+	if (Timecodes && IsSamePath(Source, Timecodes))
 		return vsapi->setError(out, "Source: Timecodes will overwrite the source");
 
-	FFMS_Index *Index = NULL;
+	FFMS_Index *Index = nullptr;
 	std::string DefaultCache;
 	if (Cache) {
 		if (CacheFile && *CacheFile) {
-			if (!_stricmp(Source, CacheFile))
+			if (IsSamePath(Source, CacheFile))
 				return vsapi->setError(out, "Source: Cache will overwrite the source");
 			Index = FFMS_ReadIndex(CacheFile, &E);
 		} else {
@@ -162,15 +193,20 @@ static void VS_CC CreateSource(const VSMap *in, VSMap *out, void *, VSCore *core
 			Index = FFMS_ReadIndex(CacheFile, &E);
 			// Reindex if the index doesn't match the file and its name wasn't
 			// explicitly given
-			if (Index && FFMS_IndexBelongsToFile(Index, Source, 0) != FFMS_ERROR_SUCCESS) {
+			if (Index && FFMS_IndexBelongsToFile(Index, Source, nullptr) != FFMS_ERROR_SUCCESS) {
 				FFMS_DestroyIndex(Index);
-				Index = 0;
+				Index = nullptr;
 			}
 		}
 	}
 
 	if (!Index) {
-		if (!(Index = FFMS_MakeIndex(Source, 0, 0, NULL, NULL, true, NULL, NULL, &E)))
+		FFMS_Indexer *Indexer = FFMS_CreateIndexer(Source, &E);
+		if (!Indexer)
+			return vsapi->setError(out, (std::string("Index: ") + E.Buffer).c_str());
+
+		Index = FFMS_DoIndexing2(Indexer, FFMS_IEH_CLEAR_TRACK, &E);
+		if (!Index)
 			return vsapi->setError(out, (std::string("Index: ") + E.Buffer).c_str());
 
 		if (Cache)
@@ -182,8 +218,10 @@ static void VS_CC CreateSource(const VSMap *in, VSMap *out, void *, VSCore *core
 
 	if (Track == -1)
 		Track = FFMS_GetFirstIndexedTrackOfType(Index, FFMS_TYPE_VIDEO, &E);
-	if (Track < 0)
+	if (Track < 0) {
+		FFMS_DestroyIndex(Index);
 		return vsapi->setError(out, "Source: No video track found");
+	}
 
 	if (Timecodes && strcmp(Timecodes, "")) {
 		if (FFMS_WriteTimecodes(FFMS_GetTrackFromIndex(Index, Track), Timecodes, &E)) {
@@ -194,38 +232,38 @@ static void VS_CC CreateSource(const VSMap *in, VSMap *out, void *, VSCore *core
 
 	VSVideoSource *vs;
 	try {
-		vs = new VSVideoSource(Source, Track, Index, FPSNum, FPSDen, Threads, SeekMode, RFFMode, Width, Height, Resizer, Format, vsapi, core);
+		vs = new VSVideoSource(Source, Track, Index, FPSNum, FPSDen, Threads, SeekMode, RFFMode, Width, Height, Resizer, Format, OutputAlpha, vsapi, core);
 	} catch (std::exception const& e) {
 		FFMS_DestroyIndex(Index);
 		return vsapi->setError(out, e.what());
 	}
 
-	vsapi->createFilter(in, out, "Source", VSVideoSource::Init, VSVideoSource::GetFrame, VSVideoSource::Free, fmSerial, 0,vs, core);
+	vsapi->createFilter(in, out, "Source", VSVideoSource::Init, VSVideoSource::GetFrame, VSVideoSource::Free, fmUnordered, 0,vs, core);
 
 	FFMS_DestroyIndex(Index);
 }
 
 static void VS_CC GetLogLevel(const VSMap *, VSMap *out, void *, VSCore *, const VSAPI *vsapi) {
-	vsapi->propSetInt(out, "level", FFMS_GetLogLevel(), 0);
+	vsapi->propSetInt(out, "level", FFMS_GetLogLevel(), paReplace);
 }
 
 static void VS_CC SetLogLevel(const VSMap *in, VSMap *out, void *, VSCore *, const VSAPI *vsapi) {
-	FFMS_SetLogLevel((int)vsapi->propGetInt(in, "level", 0, 0));
-	vsapi->propSetInt(out, "level", FFMS_GetLogLevel(), 0);
+	FFMS_SetLogLevel((int)vsapi->propGetInt(in, "level", 0, nullptr));
+	vsapi->propSetInt(out, "level", FFMS_GetLogLevel(), paReplace);
 }
 
 static void VS_CC GetVersion(const VSMap *, VSMap *out, void *, VSCore *, const VSAPI *vsapi) {
 	int Version = FFMS_GetVersion();
 	char buf[100];
 	sprintf(buf, "%d.%d.%d.%d", Version >> 24, (Version & 0xFF0000) >> 16, (Version & 0xFF00) >> 8, Version & 0xFF);
-	vsapi->propSetData(out, "version", buf, 0, 0);
+	vsapi->propSetData(out, "version", buf, -1, paReplace);
 }
 
 VS_EXTERNAL_API(void) VapourSynthPluginInit(VSConfigPlugin configFunc, VSRegisterFunction registerFunc, VSPlugin *plugin) {
 	configFunc("com.vapoursynth.ffms2", "ffms2", "FFmpegSource 2 for VapourSynth", VAPOURSYNTH_API_VERSION, 1, plugin);
-	registerFunc("Index", "source:data;cachefile:data:opt;indexmask:int:opt;dumpmask:int:opt;audiofile:data:opt;errorhandling:int:opt;overwrite:int:opt;demuxer:data:opt;", CreateIndex, NULL, plugin);
-	registerFunc("Source", "source:data;track:int:opt;cache:int:opt;cachefile:data:opt;fpsnum:int:opt;fpsden:int:opt;threads:int:opt;timecodes:data:opt;seekmode:int:opt;width:int:opt;height:int:opt;resizer:data:opt;format:int:opt;", CreateSource, NULL, plugin);
-	registerFunc("GetLogLevel", "", GetLogLevel, NULL, plugin);
-	registerFunc("SetLogLevel", "level:int;", SetLogLevel, NULL, plugin);
-	registerFunc("Version", "", GetVersion, NULL, plugin);
+	registerFunc("Index", "source:data;cachefile:data:opt;indextracks:int[]:opt;dumptracks:int[]:opt;audiofile:data:opt;errorhandling:int:opt;overwrite:int:opt;demuxer:data:opt;", CreateIndex, nullptr, plugin);
+	registerFunc("Source", "source:data;track:int:opt;cache:int:opt;cachefile:data:opt;fpsnum:int:opt;fpsden:int:opt;threads:int:opt;timecodes:data:opt;seekmode:int:opt;width:int:opt;height:int:opt;resizer:data:opt;format:int:opt;alpha:int:opt;", CreateSource, nullptr, plugin);
+	registerFunc("GetLogLevel", "", GetLogLevel, nullptr, plugin);
+	registerFunc("SetLogLevel", "level:int;", SetLogLevel, nullptr, plugin);
+	registerFunc("Version", "", GetVersion, nullptr, plugin);
 }
